@@ -1,4 +1,4 @@
-__version__ = "0.0.3"
+__version__ = "0.0.4"
 __all__ = ["Date"]
 
 import re
@@ -55,13 +55,25 @@ tokens = (
     # SPACE,
     "YESTERDAY",
     "TOMORROW",
-    "THE_DAY_AFTER_TOMORROW",
-    "THE_DAY_BEFORE_YESTERDAY",
+    "AFTER_TOMORROW",
+    "BEFORE_YESTERDAY",
     "TODAY",
     "AT",
     "ON",
+    "OF",
+    "THE",
+    "DATE_END",
+    "AM",
+    "PM",
 )
 
+
+def t_DATE_END(t):
+    r"st\b|nd\b|rd\b|th\b"
+    # print('date-end detected!', t.value)
+    return t
+
+
 # def t_SPACE(t):
 #     r"\s+"
 #     # ignore whitespace
@@ -83,6 +95,7 @@ def t_MINUS(t):
 def t_NUMBER(t):
     r"\d+"
     t.value = int(t.value)
+    # print('number detected!', t.value)
     return t
 
 
@@ -165,10 +178,12 @@ def t_WORD_NUMBER(t):
 
 t_DAY = r"monday|tuesday|wednesday|thursday|friday|saturday|sunday"
 
+t_MONTH = r"january|february|march|april|may|june|july|august|september|october|november|december"
+
 
 def t_TIME(t):
     r"years|months|weeks|days|hours|minutes|seconds|milliseconds|year|month|week|day|hour|minute|second|millisecond"
-
+    # print('time detected!', t.value)
     if t.value.endswith("s"):
         t.value = t.value[:-1]
         # TODO - set a flag to indicate this is a plural
@@ -185,13 +200,51 @@ t_PAST_PHRASE = r"today\ minus|today\ take|today\ take\ away|now\ minus|now\ tak
 
 t_YESTERDAY = r"yesterday"
 t_TOMORROW = r"tomorrow|2moro|2morro"
-t_THE_DAY_AFTER_TOMORROW = (
-    r"the day after tomorrow|the day after 2moro|the day after 2morro"
-)
-t_THE_DAY_BEFORE_YESTERDAY = r"the day before yesterday"
+# t_AFTER_TOMORROW = r"after\ tomorrow|after\ 2moro|after\ 2morro"
+def t_AFTER_TOMORROW(t):
+    r"after\ tomorrow|after\ 2moro|after\ 2morro"
+    return t
+
+
+# t_BEFORE_YESTERDAY = r"before\ yesterday|other\ day"
+def t_BEFORE_YESTERDAY(t):
+    r"before\ yesterday|other\ day"
+    # print('before yesterday detected!', t.value)
+    return t
+
+
 t_TODAY = r"today"
-t_AT = r"at|@"
+
+# t_AT = r"at|@"
+
+
+def t_AT(t):
+    r"at|@"
+    # print('@ detected!', t.value)
+    return t
+
+
 t_ON = r"on"
+t_OF = r"of"
+# t_THE = r"the"
+
+
+def t_AM(t):
+    r"am"
+    # print('am:morning detected!', t.value)
+    return t
+
+
+def t_PM(t):
+    r"pm"
+    # print('pm:afternoon detected!', t.value)
+    return t
+
+
+def t_THE(t):
+    r"the"
+    # print('the detected!', t.value)
+    return t
 
 
 t_YEAR = r"\d{4}"
@@ -372,6 +425,10 @@ def p_date(p):
     date_list : date_yesterday
     date_list : date_2moro
     date_list : date_day
+    date_list : date_end
+    date_list : date_or
+    date_list : date_before_yesterday
+    date_list : date_after_tomorrow
     """
     p[0] = [p[1]]
 
@@ -384,6 +441,10 @@ def p_single_date(p):
     """
     date : TIME
     date : NUMBER TIME
+    date : NUMBER AM
+    date : NUMBER PM
+    date : AT NUMBER AM
+    date : AT NUMBER PM
     date : WORD_NUMBER TIME
     date : PHRASE TIME
     date : TIME PHRASE
@@ -394,9 +455,52 @@ def p_single_date(p):
     if len(p) == 2:
         p[0] = DateFactory(p[1], 1)
     elif len(p) == 3:
+        # print('bb', p[1], p[2])
+        if isinstance(p[1], int):
+            # 5-pm
+            if p[2] == "am":
+                if p[1] == 12:
+                    p[1] = 0
+                params = {
+                    "hour": p[1],
+                    "minute": 0,
+                    "second": 0,
+                }
+                p[0] = DateFactory.create_date(**params)
+            elif p[2] == "pm":
+                params = {
+                    "hour": p[1],
+                    "minute": 0,
+                    "second": 0,
+                }
+                p[0] = DateFactory.create_date(**params)
+            else:  # number time
+                params = {p[2]: p[1]}
+                p[0] = DateFactory.create_date_with_offsets(**params)  # '3 days'
+            return
         params = {p[2]: 1}  # TODO - prepend offset_ to the key. passing 1 as no number
         p[0] = DateFactory.create_date_with_offsets(**params)  # 'In a minute'
     elif len(p) == 4:
+        # print("here we are", p[1], p[2], p[3])
+        if p[1] == "at" or p[1] == "@":
+            # at-3-am
+            if p[3] == "am":
+                if p[2] == 12:
+                    p[2] = 0
+                params = {
+                    "hour": p[2],
+                    "minute": 0,
+                    "second": 0,
+                }
+                p[0] = DateFactory.create_date(**params)
+            elif p[3] == "pm":
+                params = {
+                    "hour": p[2],
+                    "minute": 0,
+                    "second": 0,
+                }
+                p[0] = DateFactory.create_date(**params)
+            return
         if p[1] == "an":
             p[1] = 1  # if no number is passed, assume 1
         params = {p[2]: p[1]}  # TODO - prepend offset_ to the key
@@ -521,12 +625,105 @@ def p_single_date_day(p):
         p[0] = d
 
 
-# t_THE_DAY_AFTER_TOMORROW = r"the day after tomorrow|the day after 2moro|the day after 2morro"
-# t_THE_DAY_BEFORE_YESTERDAY = r"the day before yesterday"
-# t_TODAY = r"today"
-# t_AT = r"at|@"
-# t_ON = r"on"
+def p_this_or_next_period(p):
+    """
+    date_or : PAST_PHRASE TIME
+    """
+    if len(p) == 3:
+        d = stDate()
+        if p[1] == "last":
+            if p[2] == "week":
+                d.set_date(d.get_date() - 7)
+            elif p[2] == "year":
+                d.set_year(d.get_year() - 1)
+            elif p[2] == "month":
+                # print(d.get_month())
+                d.set_month(
+                    d.get_month()
+                )  # -1)  #??? not sure why this is not -1???????. this must be a bug in set_month?
+                # print(d.get_month())
+            # elif p[2] == "century":
+            #     d.set_year(d.get_year() - 100)
+        elif p[1] == "next":
+            d.set_date(d.get_date() + 1)
+        p[0] = d
 
+
+def p_before_yesterday(p):
+    """
+    date_before_yesterday : BEFORE_YESTERDAY
+    date_before_yesterday : THE BEFORE_YESTERDAY
+    date_before_yesterday : THE TIME BEFORE_YESTERDAY
+    """
+    # print('before_yesterday decteted')
+    d = stDate()
+    d.set_date(d.get_date() - 2)
+    p[0] = d
+
+
+def p_after_tomorrow(p):
+    """
+    date_after_tomorrow : AFTER_TOMORROW
+    date_after_tomorrow : THE TIME AFTER_TOMORROW
+    """
+    d = stDate()
+    d.set_date(d.get_date() + 2)
+    p[0] = d
+
+
+# date_end : THE NUMBER ?? allow
+def p_single_date_end(p):
+    """
+    date_end : NUMBER DATE_END
+    date_end : THE NUMBER DATE_END
+    date_end : MONTH NUMBER DATE_END
+    date_end : NUMBER DATE_END OF MONTH
+    date_end : ON THE NUMBER DATE_END
+    date_end : MONTH THE NUMBER DATE_END
+    date_end : THE NUMBER DATE_END OF MONTH
+    """
+    if len(p) == 3:
+        # print('p:', p[1], p[2])
+        d = stDate()
+        d.set_date(p[1])
+        p[0] = d
+    if len(p) == 4:
+        # print('p-:', p[1], p[2], p[3])
+        d = stDate()
+        d.set_date(p[2])
+        # print(p[1], "the", p[1] == "the")
+        if p[1] == "the":  # the-2-nd
+            d.set_date(p[2])
+        else:  # january-14-th
+            # print('y!', p[1], p[2])
+            m = d.get_month_index_by_name(p[1])
+            # print('THE MONTH IS:',m)
+            d.set_month(m)
+            d.set_date(p[2])
+        p[0] = d
+    if len(p) == 5:
+        # print('p--:', p[1], p[2], p[3], p[4])
+        d = stDate()
+        if p[1] == "on":  # on-the-1-st
+            d.set_date(p[3])
+        else:  # april-the-1-st
+            # print('i think we are here::' , p[1], p[2], p[3], p[4])
+            m = d.get_month_index_by_name(p[1])
+            d.set_month(m)
+            # print('THE date is:', p[3])
+            d.set_date(p[3])
+            # print('THE date isaaa:', d.get_date())
+        p[0] = d
+    if len(p) == 6:
+        # print('p++:', p[1], p[2], p[3], p[4], p[5])
+        d = stDate()  # the-18-th-of-january
+        m = d.get_month_index_by_name(p[5])
+        d.set_month(m)
+        d.set_date(p[2])
+        p[0] = d
+
+
+# t_TODAY = r"today"
 # "SAME TIME ON" # TODO----
 
 
@@ -549,12 +746,16 @@ def is_now(phrase):
         "now",
         "right now",
         "right away",
+        "right this second",
+        "right this minute",
         "immediately",
         "straight away",
         "at once",
         "as soon as possible",
+        "this current moment",
         "asap",
         "here and now",
+        "today",  # this one also requires a token. i.e. today at 5pm
     ]
 
 
@@ -583,9 +784,22 @@ def replace_short_words(phrase):
     phrase = phrase.replace("wks", "week")
     phrase = phrase.replace("yr", "year")
     phrase = phrase.replace("yrs", "year")
-    phrase = phrase.replace("ms", "millisecond")
-    phrase = phrase.replace("mil", "millisecond")
-    phrase = phrase.replace("mils", "millisecond")
+    # phrase = phrase.replace("ms", "millisecond")
+    # phrase = phrase.replace("mil", "millisecond")
+    phrase = re.sub(r"\bms\b", "millisecond", phrase)
+    phrase = re.sub(r"\bmil\b", "millisecond", phrase)
+    phrase = re.sub(r"\bmils\b", "millisecond", phrase)
+    # phrase = phrase.replace("mils", "millisecond")
+
+    phrase = phrase.replace("century", "100 years")
+    phrase = phrase.replace("centuries", "100 years")
+    phrase = phrase.replace("decade", "10 years")
+    phrase = phrase.replace("decades", "10 years")
+    # phrase = phrase.replace("millenium", "1000 years")
+    # phrase = phrase.replace("millenia", "1000 years")
+    phrase = re.sub(r"\bmillenium\b", "1000 years", phrase)
+    phrase = re.sub(r"\bmillennium\b", "1000 years", phrase)
+    phrase = re.sub(r"\bmillenia\b", "1000 years", phrase)
 
     phrase = re.sub(r"\bmon\b", "monday", phrase)
     phrase = re.sub(r"\btues\b", "tuesday", phrase)
@@ -599,10 +813,29 @@ def replace_short_words(phrase):
     phrase = re.sub(r"\bsat\b", "saturday", phrase)
     phrase = re.sub(r"\bsun\b", "sunday", phrase)
 
+    phrase = re.sub(r"\bjan\b", "january", phrase)
+    phrase = re.sub(r"\bfeb\b", "february", phrase)
+    phrase = re.sub(r"\bmar\b", "march", phrase)
+    phrase = re.sub(r"\bapr\b", "april", phrase)
+    phrase = re.sub(r"\bmay\b", "may", phrase)
+    phrase = re.sub(r"\bjun\b", "june", phrase)
+    phrase = re.sub(r"\bjul\b", "july", phrase)
+    phrase = re.sub(r"\baug\b", "august", phrase)
+    phrase = re.sub(r"\bsept\b", "september", phrase)
+    phrase = re.sub(r"\bsep\b", "september", phrase)
+    phrase = re.sub(r"\boct\b", "october", phrase)
+    phrase = re.sub(r"\bnov\b", "november", phrase)
+    phrase = re.sub(r"\bdec\b", "december", phrase)
+
+    # special cases
+    phrase = phrase.replace("a few", "3")
+    # phrase = re.sub(r"a few\b", "3", phrase)
+    # phrase = re.sub(r"\bseveral\b", "7", phrase)
+
     return phrase
 
 
-def Date(date, *args, **kwargs):
+def get_date(date, *args, **kwargs):
     try:
         phrase = date.lower()
         phrase = phrase.strip()
@@ -610,7 +843,41 @@ def Date(date, *args, **kwargs):
         if is_now(phrase):
             return stDate()
         return yacc.parse(phrase)[0]
-    except TypeError:
+    except TypeError as e:
+        # if debug raise the error
+        if DEBUG:
+            raise e
         return stDate(date, *args, **kwargs)
     except Exception as e:
+        if DEBUG:
+            raise e
         return stDate()
+
+
+def Date(date=None, *args, length: int = None, **kwargs):
+    """
+    # if 2nd argument is a string its a date range
+    # if a length is passed, and there's a range, then we need to split it
+    # by filling and array with dates between the range
+    # print(date, args)
+    if len(args) > 0 and isinstance(args[0], str):
+        first_date = get_date(date, *args, **kwargs)
+        second_date = get_date(args[0], *args, **kwargs)
+        if length:
+            # return [first_date + i for i in range(length)]
+            # get the difference between the two dates
+            diff = second_date.get_time() - first_date.get_time()
+            # divide by the length
+            diff = diff / length
+            # populate an array with the dates
+            dates = []
+            dates.append(first_date)
+            for i in range(length-2):
+                d = Date('now')
+                d.set_seconds(d.seconds + (diff/1000))
+                dates.append(d)
+            dates.append(second_date)
+            return dates
+        return [first_date, second_date]
+    """
+    return get_date(date)
